@@ -1,6 +1,8 @@
+[ -z "$UPSTREAM_PULL_REQUEST" ] && { echo "Skipping test, not ran as part of a pull request."; exit 0; }
+
 sudo snap install core
 
-snapcraft
+sudo snapcraft
 snapfile=`find . -maxdepth 1 -name \*.snap`
 
 sudo snap install $snapfile --classic --dangerous
