@@ -10,7 +10,7 @@ import contextlib
 from contextlib import ExitStack, contextmanager
 from distutils.spawn import find_executable
 from parted import Device
-from subprocess import PIPE, run as subprocess_run
+from subprocess import DEVNULL, PIPE, run as subprocess_run
 from tempfile import NamedTemporaryFile, TemporaryDirectory
 from ubuntu_image.state import ExpectedError
 
@@ -25,6 +25,7 @@ __all__ = [
     'run',
     'snap',
     'sparse_copy',
+    'unsparse_swapfile_ext4',
     ]
 
 
@@ -209,6 +210,17 @@ def mount(img):
         yield mountpoint
 
 
+def unsparse_swapfile_ext4(img_file):
+    # A workaround for mkfs.ext4 messing up our rootfs swapfiles by turning
+    # them to sparse files without us asking.
+    with mount(img_file) as mountpoint:
+        swapfile_path = os.path.join(mountpoint, 'swapfile')
+        if os.path.exists(swapfile_path):
+            cmd = ('dd if={swapfile} of={swapfile} conv=notrunc '
+                   'bs=1M'.format(swapfile=swapfile_path))
+            run(cmd, stdout=DEVNULL, stderr=DEVNULL)
+
+
 def mkfs_ext4(img_file, contents_dir, image_type, label='writable',
               preserve_ownership=False):
     """Encapsulate the `mkfs.ext4` invocation.
