@@ -196,7 +196,7 @@ GadgetYAML = Schema({
         Match('^[a-zA-Z0-9][-a-zA-Z0-9]*$'): Schema({
             Optional('schema', default='gpt' if has_new_voluptuous()
                      else VolumeSchema.gpt):
-                Enumify(VolumeSchema),
+            Enumify(VolumeSchema),
             Optional('bootloader'): Enumify(
                 BootLoader, preprocessor=methodcaller('replace', '-', '')),
             Optional('id'): Coerce(Id),
@@ -213,7 +213,7 @@ GadgetYAML = Schema({
                 Optional('id'): Coerce(UUID),
                 Optional('filesystem', default='none' if has_new_voluptuous()
                          else FileSystemType.none):
-                    Enumify(FileSystemType),
+                Enumify(FileSystemType),
                 Optional('filesystem-label'): str,
                 Optional('content'): Any(
                     [Schema({
@@ -227,8 +227,7 @@ GadgetYAML = Schema({
                         Optional('offset-write'): Any(
                             Coerce(Size32bit), RelativeOffset),
                         Optional('size'): Coerce(as_size),
-                        })
-                    ],
+                        })],
                 ),
                 Optional('update'): Schema({
                     Optional('edition'): All(
@@ -500,7 +499,9 @@ def parse(stream_or_string):
                 rootfs_seen = True
                 # For images to work the system-data (rootfs) partition needs
                 # to have the 'writable' filesystem label set.
-                if filesystem_label not in (None, 'writable'):
+                # For UC20 this requirement no longer stands.
+                if (filesystem_label not in (None, 'writable') and
+                        not is_seeded):
                     raise GadgetSpecificationError(
                         '`role: system-data` structure must have an implicit '
                         "label, or 'writable': {}".format(filesystem_label))
@@ -550,10 +551,6 @@ def parse(stream_or_string):
                 structure_type, structure_id, structure_role,
                 filesystem, filesystem_label,
                 content_specs))
-            # If we found a system-seed partition, stop looking at other
-            # parts.
-            if is_seeded:
-                break
         # Sort structures by their offset.
         volume_specs[image_name] = VolumeSpec(
             schema, bootloader, image_id, structures)
