@@ -64,6 +64,7 @@ class TestClassicBuilder(TestCase):
         # even if gadget tree is placed locally on the machine.
         workdir = self._resources.enter_context(TemporaryDirectory())
         args = SimpleNamespace(
+            cmd='classic',
             project='ubuntu-cpc',
             suite='xenial',
             arch='amd64',
@@ -78,6 +79,7 @@ class TestClassicBuilder(TestCase):
             extra_ppas=None,
             hooks_directory=[],
             disk_info=None,
+            disable_console_conf=False,
             gadget_tree=self.gadget_tree,
             filesystem=None,
             )
@@ -107,6 +109,7 @@ class TestClassicBuilder(TestCase):
         unpackdir = os.path.join(workdir, 'unpack')
         mock = LiveBuildMocker(unpackdir)
         args = SimpleNamespace(
+            cmd='classic',
             project='ubuntu-cpc',
             suite='xenial',
             arch='amd64',
@@ -121,6 +124,7 @@ class TestClassicBuilder(TestCase):
             extra_ppas='ppa:some/ppa',
             hooks_directory=[],
             disk_info=None,
+            disable_console_conf=False,
             gadget_tree=self.gadget_tree,
             filesystem=None,
             )
@@ -152,6 +156,7 @@ class TestClassicBuilder(TestCase):
         with ExitStack() as resources:
             workdir = resources.enter_context(TemporaryDirectory())
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -166,6 +171,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -209,6 +215,7 @@ class TestClassicBuilder(TestCase):
         with ExitStack() as resources:
             workdir = resources.enter_context(TemporaryDirectory())
             args = SimpleNamespace(
+                cmd='classic',
                 project=None,
                 suite='xenial',
                 arch='amd64',
@@ -223,6 +230,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -266,6 +274,7 @@ class TestClassicBuilder(TestCase):
         with ExitStack() as resources:
             workdir = resources.enter_context(TemporaryDirectory())
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -280,6 +289,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -325,6 +335,7 @@ class TestClassicBuilder(TestCase):
                 NamedTemporaryFile('w', encoding='utf-8'))
             print('cloud init user data', end='', flush=True, file=cloud_init)
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -339,6 +350,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -384,6 +396,7 @@ class TestClassicBuilder(TestCase):
                 NamedTemporaryFile('w', encoding='utf-8'))
             print('cloud init user data', end='', flush=True, file=cloud_init)
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -398,6 +411,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -445,6 +459,7 @@ class TestClassicBuilder(TestCase):
         with ExitStack() as resources:
             workdir = resources.enter_context(TemporaryDirectory())
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -459,6 +474,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -512,6 +528,7 @@ class TestClassicBuilder(TestCase):
             unpackdir = resources.enter_context(TemporaryDirectory())
             # Fast forward a state machine to the method under test.
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -527,6 +544,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -604,6 +622,7 @@ class TestClassicBuilder(TestCase):
             workdir = resources.enter_context(TemporaryDirectory())
             # Fast forward a state machine to the method under test.
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -619,6 +638,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -656,6 +676,7 @@ class TestClassicBuilder(TestCase):
             unpackdir = resources.enter_context(TemporaryDirectory())
             # Fast forward a state machine to the method under test.
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -672,6 +693,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -718,6 +740,7 @@ class TestClassicBuilder(TestCase):
             unpackdir = resources.enter_context(TemporaryDirectory())
             # Fast forward a state machine to the method under test.
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -734,6 +757,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -825,6 +849,7 @@ class TestClassicBuilder(TestCase):
             unpackdir = resources.enter_context(TemporaryDirectory())
             # Fast forward a state machine to the method under test.
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -841,6 +866,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -874,6 +900,7 @@ class TestClassicBuilder(TestCase):
             unpackdir = resources.enter_context(TemporaryDirectory())
             # Fast forward a state machine to the method under test.
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -890,6 +917,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -930,10 +958,12 @@ class TestClassicBuilder(TestCase):
                 'with_proposed': 'PROPOSED',
                 }
             kwargs_skel = {
+                'cmd': 'classic',
                 'workdir': '/tmp',
                 'output_dir': '/tmp',
                 'hooks_directory': '/tmp',
                 'disk_info': None,
+                'disable_console_conf': False,
                 'output': None,
                 'cloud_init': None,
                 'gadget_tree': None,
@@ -1003,6 +1033,7 @@ class TestClassicBuilder(TestCase):
                 'output_dir': '/tmp',
                 'hooks_directory': '/tmp',
                 'disk_info': None,
+                'disable_console_conf': False,
                 'output': None,
                 'cloud_init': None,
                 'gadget_tree': None,
@@ -1040,6 +1071,7 @@ class TestClassicBuilder(TestCase):
             outputdir = resources.enter_context(TemporaryDirectory())
             # Fast forward a state machine to the method under test.
             args = SimpleNamespace(
+                cmd='classic',
                 project='ubuntu-cpc',
                 suite='xenial',
                 arch='amd64',
@@ -1056,6 +1088,7 @@ class TestClassicBuilder(TestCase):
                 extra_ppas=None,
                 hooks_directory=[],
                 disk_info=None,
+                disable_console_conf=False,
                 gadget_tree=self.gadget_tree,
                 filesystem=None,
                 )
@@ -1091,3 +1124,73 @@ class TestClassicBuilder(TestCase):
                            bar 3.12.3-0ubuntu1
                            baz 2.3
                            """))
+
+    def test_workaround_sparse_swapfile(self):
+        # Test if the swapfile unsparsing workaround is fired.
+        with ExitStack() as resources:
+            workdir = resources.enter_context(TemporaryDirectory())
+            unpackdir = resources.enter_context(TemporaryDirectory())
+            # Fast forward a state machine to the method under test.
+            args = SimpleNamespace(
+                cmd='classic',
+                project='ubuntu-cpc',
+                suite='xenial',
+                arch='amd64',
+                image_format='img',
+                unpackdir=unpackdir,
+                workdir=workdir,
+                debug=None,
+                cloud_init=None,
+                output=None,
+                subproject=None,
+                subarch=None,
+                output_dir=None,
+                with_proposed=None,
+                extra_ppas=None,
+                hooks_directory=[],
+                disk_info=None,
+                disable_console_conf=False,
+                gadget_tree=self.gadget_tree,
+                filesystem=None,
+                )
+            # Jump right to the method under test.
+            state = resources.enter_context(XXXClassicBuilder(args))
+            state._next.pop()
+            state._next.append(state.populate_filesystems)
+            # Set up expected state.
+            state.unpackdir = unpackdir
+            state.images = os.path.join(workdir, '.images')
+            os.makedirs(state.images)
+            part0_img = os.path.join(state.images, 'part0.img')
+            # Craft a gadget specification.
+            part = SimpleNamespace(
+                role=StructureRole.system_data,
+                filesystem=FileSystemType.ext4,
+                filesystem_label='writable',
+                size=150,
+                )
+            volume = SimpleNamespace(
+                structures=[part],
+                bootloader=BootLoader.grub,
+                )
+            state.gadget = SimpleNamespace(
+                volumes=dict(volume1=volume),
+                seeded=False,
+                )
+            prep_state(state, workdir, [part0_img])
+            # Prepare the dummy swapfile on the rootfs
+            state.rootfs = os.path.join(workdir, 'rootfs')
+            os.makedirs(state.rootfs)
+            swap_path = os.path.join(state.rootfs, 'swapfile')
+            open(swap_path, 'w')
+            # Mock out the mkfs.ext4 call, and we'll just test the contents
+            # directory (i.e. what would go in the ext4 file system).
+            mock_mkfs = resources.enter_context(
+                patch('ubuntu_image.common_builder.mkfs_ext4'))
+            mock_unsparse = resources.enter_context(
+                patch('ubuntu_image.common_builder.unsparse_swapfile_ext4'))
+            next(state)
+            # Check that mkfs.ext4 got called, after which we tried to
+            # unsparse the swapfile as expected.
+            self.assertEqual(len(mock_mkfs.call_args_list), 1)
+            self.assertEqual(len(mock_unsparse.call_args_list), 1)
